use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Arthur
Axel
BooK
Book
Bruhat
Christian
Contextual
Dan
Default
Easy
Etheridge
Florian
Fredric
Graham
HasLogger
Jakob
Karen
Kent
Knop
Log
Malone
Matt
Package
Peter
Philippe
Rabbitson
Riddle
Role
Router
Schlichtin
Schmidt
SetLogger
SimpleLogger
TeeLogger
Trout
Tyler
Voss
Walde
WarnLogger
Wes
WithLogger
book
ether
fREW
frioux
fsfs
grinnz
haarg
kentfredric
lib
mst
ribasushi
riddle
voss
walde
wes
